import struct
import snap7
from snap7 import util


class DTbool:
	def __init__(self, readBuffer,boolIndex):
		self.readBuffer=readBuffer
		self.boolIndex=boolIndex
		


	def readValue (self,offset):		
		
		result=snap7.util.get_bool(self.readBuffer,offset,self.boolIndex)
		return result

	


import struct
import snap7

class DTint:
	def __init__(self, readBuffer):
		self.readBuffer=readBuffer


	def db_readInt(self,byteArray,index):
		data=byteArray[index:(index+2)]
		value=struct.unpack('>h' ,struct.pack('2B',*data))[0]
		return value

	def readValue (self,offset):		
		
		
		result=self.db_readInt(self.readBuffer,offset)
		return result



#-*- coding: utf-8 -*-
import snap7
import sys
from EasyS7.Utility import *
from EasyS7.DataBlock import *
from datetime import datetime


class PLC:

    def __init__(self,plc_address , plc_rack , plc_slot , plc_tcpport=102):
        print("[INFO] : Initialization started...")
        self.plc_address = str(plc_address)
        self.plc_rack = int(plc_rack)
        self.plc_slot = int(plc_slot)
        self.plc_tcpport = plc_tcpport
        self.plc = snap7.client.Client()
        self.db_items = None
        self.db_size=None
        print("[INFO] : Initialization done...")



    def getDbItems():
        return self.db_items

    def  getDbSize():
        return self.db_size

    def setPlcAdress(plc_address):
        self.plc_address = str(plc_address)

    def setPlcRack(plc_rack):
        self.plc_rack = plc_rack

    def setPlcSlot(plc_slot):
        self.plc_slot = plc_slot

    def setPlcTcpport(plc_tcpport):
        self.plc_tcpport = plc_tcpport

    def disconnect(self):
        print("[INFO] : Closing PLC connection...")
        self.plc.disconnect()
        if not self.plc.get_connected():
                print("[INFO] : Connection closed.")
        else:
                print("[ERROR] : Connection close failed.")
                sys.exit()

    def connect(self):
        print("[INFO] : Establishing PLC connection...")
        try:

            self.plc.connect(self.plc_address,self.plc_rack,self.plc_slot,self.plc_tcpport)
            if self.plc.get_connected():
                print("[INFO] : Connection succeed.")
            else:
                print("[ERROR] : Connection failed.")
                sys.exit()

        except Exception as e:

            print("[ERROR] : ",str(e))
            print("[ERROR] : Connection failed.")
            sys.exit()


    def readDb(self,db_no):
            print("[INFO] : Data block read operation started...")
            if self.db_items == None or self.db_size == None:
                print("[ERROR] : Template Info Null, Use readTemplate() First To Fill Template Info")
                sys.exit()
            else:
                db_data = dbRead(self.plc,db_no,self.db_size,self.db_items).__dict__
                print("[INFO] : Data block read operation done...")
                return db_data

    def readTemplate(self, layout_path):
            self.db_items = getItemsFromDbLayout(layout_path)
            self.db_size = getDbSize(self.db_items,'bytebit','datatype')
            




    """def writeDb(self,db_no,data):
        print("[INFO] : Data block write operation started...")
        if self.db_items == None or self.db_size == None:
            print("[ERROR] : Template Info Null, Use readTemplate() First To Fill Template Info")
            sys.exit()
        else:
            for item in data:
                pass"""
                        
    def writeArea(self,area_type, address ,item_data_type, item, db_num = 999, bool_index = -1, string_max_size = -1):
        dbWriteArea(self.plc,area_type, address ,item_data_type, item, db_num,  bool_index , string_max_size )
        #dat = self.plc.read_area(snap7.snap7types.S7AreaDB,101,0,4)
        
    def readArea(self,area_type, address ,item_data_type, db_num = 999, bool_index = -1, string_max_size = -1):
         return dbReadArea(self.plc,area_type, address ,item_data_type, db_num,  bool_index , string_max_size )
        #dat = self.plc.read_area(snap7.snap7types.S7AreaDB,101,0,4)

        # burası duzenlenecek ve dıger data typlar eklenecek. Bool olanı tekrardan kontrol et


#-*- coding: utf-8 -*-

import snap7
import random
import sys
from datetime import datetime
from enum import Enum

from EasyS7.Utility import *
from EasyS7.DataTypes.DTdint import  DTdint
from EasyS7.DataTypes.DTbool import  DTbool
from EasyS7.DataTypes.DTtime import  DTtime
from EasyS7.DataTypes.DTint import  DTint
from EasyS7.DataTypes.DTstring import  DTstring
from decimal import Decimal as D

class Areas(Enum):
    Input = 1
    Output = 2
    Merker = 3
    DB = 4
    Counter = 5
    Timer = 6

class DataTypes(Enum):
    Real = 1
    Bool = 2
    DInt = 3
    UDInt = 4
    Int = 5
    DTime = 6
    String = 7
    Multiple = 8


class DataBlockObj(object):
		pass





def dbRead(plc,db_num,length,dbitems):

    data=plc.db_read(db_num,0,length)
    obj = DataBlockObj()

    for item in dbitems:
        value = (None,item['name'])
        offset = int(item['bytebit'].split('.')[0])

        if item['datatype']=='Real':
            value = (snap7.util.get_real(data,offset),item['name'].replace(" ","_").replace("/","_"))
            obj.__setattr__(item['name'].replace(" ","_").replace("/","_"), value[0])

        if item['datatype']=='Bool':
            bit =int(item['bytebit'].split('.')[1])
            BoolObj=DTbool(data,bit)
            value = (snap7.util.get_bool(data,offset,bit),item['name'].replace(" ","_").replace("/","_"))
            obj.__setattr__(item['name'].replace(" ","_").replace("/","_"), value[0])

        if item['datatype']=='DInt' :

            DintObj=DTdint(data)
            value=(DintObj.readValue(offset),item['name'].replace(" ","_").replace("/","_"))
            obj.__setattr__(item['name'].replace(" ","_").replace("/","_"), value[0])

        if item['datatype']=='UDInt':

            DintObj=DTdint(data)
            value=(DintObj.readValueU(offset),item['name'].replace(" ","_").replace("/","_"))
            obj.__setattr__(item['name'].replace(" ","_").replace("/","_"), value[0])

        if item['datatype']=='Int':

            IntObj=DTint(data)
            value=(IntObj.readValue(offset),item['name'].replace(" ","_").replace("/","_"))
            obj.__setattr__(item['name'].replace(" ","_").replace("/","_"), value[0])


        """if item['datatype']=='Time':

            TimeObj=DTtime() # burası düzeltilecek
            value=(TimeObj.readValue(offset),item['name'].replace(" ","_").replace("/","_"))
            obj.__setattr__(item['name'].replace(" ","_").replace("/","_"), value[0])"""



        if item['datatype'].startswith('String'):
            value = snap7.util.get_string(data, offset,int(item['datatype'].split('[')[1][:-1])+1)
            obj.__setattr__(item['name'].replace(" ","_").replace("/","_"), value)

        #obj.__setattr__(item['name'].replace(" ","_").replace("/","_"), value[0])


    return obj


def dbWrite(plc,itemArray): # {data_type, offset, data,}
    print(itemArray)
    offsets = { "Bool":1,"Int": 2,"Real":4,"Time":4,"DInt":4,"UDInt":4}#yeni offset degerleri eklenicek
    seq,length = [x["offset"] for x in itemArray],[x["data_type"] for x in itemArray]

    maximum=max(seq, key=float)
    minimum = min(seq, key=float)

    idx = seq.index(maximum)
    if length[idx].startswith('String'):

        lastByte = int(maximum)+(int(length[idx].split('[')[1][:-1]))

    else:

        lastByte = int(maximum)+(offsets[length[idx]])
           

    byte_array=bytearray(lastByte-int(minimum))
    
    print(byte_array)
    
    for item in itemArray:

        if item['data_type']=='Real':
            
            snap7.util.set_real(byte_array,int(item["offset"]-int(minimum)),item["data"])

        elif item['data_type']=='Bool':
                
                
                bool_index = int((D(str(item["offset"])) - D(str(int(item["offset"])))) * 10)
                print(bool_index)
                print(int(item["offset"]-int(minimum)))
                
                snap7.util.set_bool(byte_array,int(item["offset"]-int(minimum)),bool_index,item["data"])

        elif item['data_type']=='DInt' :
            
            DIntObj = DTdint(byte_array)
            byte_array = DIntObj.set_dint(byte_array,int(item["offset"]-int(minimum)),item["data"])

        elif item['data_type']=='UDInt':
            
            DIntObj = DTdint(byte_array)
            byte_array = DIntObj.set_udint(byte_array,int(item["offset"]-int(minimum)),item["data"])

        elif item['data_type']=='Int':
            
            snap7.util.set_int(byte_array,int(item["offset"]-int(minimum)),item["data"])

    return byte_array


def dbReadArea(plc,area_type, address ,item_data_type,  db_num = 999, bool_index = -1, string_max_size = -1):
    address_integer = int(address)
    address_fraction = (address-address_integer)*10
    value = None

    if area_type == Areas.Input:
        area = snap7.snap7types.S7AreaPE
        offset = address_integer*8 + address_fraction
    elif area_type == Areas.Output:
        area = snap7.snap7types.S7AreaPA
        offset = address_integer*8 + address_fraction
    elif area_type == Areas.Merker:
        area = snap7.snap7types.S7AreaMK
        offset = address_integer*8 + address_fraction
    elif area_type == Areas.DB:
        if db_num == 999:
            print("[Error] : Data Block Number Not Defined. Use Optional Argument db_num.")
            sys.exit()
        area = snap7.snap7types.S7AreaDB
        offset = address
    elif area_type == Areas.Counter:
        area = snap7.snap7types.S7AreaCT
        offset = address_integer*16 + address_fraction
    elif area_type == Areas.Timer:
        area = snap7.snap7types.S7AreaTM
        offset = address_integer*16 + address_fraction



    if item_data_type == DataTypes.Real:
        byte_array = plc.read_area(area,db_num,offset,4)
        value = snap7.util.get_real(byte_array,0)
        

    elif item_data_type == DataTypes.Bool:
        if bool_index <0 : 
            print("[Error] : Bool Index Not Defined. Use Optional Argument bool_index")
            sys.exit()
        else:
            byte_array = plc.read_area(area,db_num,offset,1)
            value = snap7.util.get_bool(byte_array,0,bool_index)

    elif item_data_type == DataTypes.DInt:

        byte_array = plc.read_area(area,db_num,offset,4)
        DTdintObj = DTdint(byte_array)
        value = DTdintObj.db_readDint(byte_array,0 )
        

    elif item_data_type == DataTypes.UDInt:
        byte_array = plc.read_area(area,db_num,offset,4)
        value=struct.unpack('>L' ,struct.pack('4B',*byte_array))[0]

    elif item_data_type == DataTypes.Int:
        byte_array = plc.read_area(area,db_num,offset,2)
        value = snap7.util.get_int(byte_array,0)

    elif item_data_type == DataTypes.DTime:

        print("[ERROR] : Time Not Implemented")
        sys.exit()

        
        

    elif item_data_type == DataTypes.String:
        if string_max_size <= 0 :
            print("[Error] : Max String Size Not Defined. Use Optional Argument string_max_size")
            sys.exit()
        else:
            
            
            byte_array = plc.read_area(area,db_num,offset,string_max_size)
            
            value = snap7.util.get_string(byte_array,0,string_max_size)

    return value




def dbWriteArea(plc,area_type, address ,item_data_type, item, db_num = 999, bool_index = -1, string_max_size = -1):

    address_integer = int(address)
    address_fraction = (address-address_integer)*10
    byte_array = None

    if area_type == Areas.Input:
        area = snap7.snap7types.S7AreaPE
        #offset = address_integer*8 + address_fraction
    elif area_type == Areas.Output:
        area = snap7.snap7types.S7AreaPA
        #offset = address_integer*8 + address_fraction
    elif area_type == Areas.Merker:
        area = snap7.snap7types.S7AreaMK
        #offset = address_integer*8 + address_fraction
    elif area_type == Areas.DB:
        if db_num == 1:
            print("[Error] : Data Block Number Not Defined. Use Optional Argument db_num.")
            sys.exit()
        area = snap7.snap7types.S7AreaDB
        #offset = address
    elif area_type == Areas.Counter:
        area = snap7.snap7types.S7AreaCT
        #offset = address_integer*16 + address_fraction
    elif area_type == Areas.Timer:
        area = snap7.snap7types.S7AreaTM
        #offset = address_integer*16 + address_fraction

    
    offset = int(address)
    
    if item_data_type == DataTypes.Real:
        byte_array = bytearray(4)
        snap7.util.set_real(byte_array,0,item)

    elif item_data_type == DataTypes.Bool:
        if bool_index == 999 : 
            print("[Error] : Bool Index Not Defined. Use Optional Argument bool_index")
            sys.exit()
        else:
           
            byte_array = plc.read_area(area,db_num,offset,1)
            snap7.util.set_bool(byte_array,0,bool_index,item)

    elif item_data_type == DataTypes.DInt:
        byte_array = bytearray(4)
        DIntObj = DTdint(byte_array)
        byte_array = DIntObj.set_dint(byte_array,0,item)

    elif item_data_type == DataTypes.UDInt:
        byte_array = bytearray(4)
        DIntObj = DTdint(byte_array)
        byte_array = DIntObj.set_udint(byte_array,0,item)

    elif item_data_type == DataTypes.Int:
        byte_array = bytearray(4)
        snap7.util.set_int(byte_array,0,item)

    elif item_data_type == DataTypes.DTime:

        print("[ERROR] : Time Not Implemented")
        sys.exit()
        """dtl = plc.db_read(db_num,16, 8)
        
        pdb.set_trace()
        
        DTimeObject = DTtime()
        byte_array1 = (DTimeObject.set_dtime(item))
        byte_array = dtl
        pdb.set_trace()"""
        
        

    elif item_data_type == DataTypes.String:
        if string_max_size <= 0 :
            print("[Error] : Max String Size Not Defined. Use Optional Argument string_max_size")
            sys.exit()
        else:
            
            StringObj = DTstring()
            byte_array = plc.read_area(area,db_num,offset,string_max_size)

            StringObj.set_string(byte_array,0,item,string_max_size)

    elif item_data_type == DataTypes.Multiple:
        byte_array = dbWrite(plc,item)

        
        


          
    
   

    plc.write_area(area,db_num,offset,byte_array)





import struct
import snap7
from datetime import datetime
from ctypes import  c_int32
import pdb


class DTtime:
    def __init__(self):
        pass


    def db_readTime(self,byteArray,index):
        data=byteArray[index:(index+4)]
        value=struct.unpack('>l' ,struct.pack('4B',*data))[0]
        return value

    def readValue (self,offset):        
        
        
        result=self.db_readTime(self.readBuffer,offset)
        return result

    def set_dtime(self,dt):

        



        type_ = c_int32
        buf = bytearray(8)
        buf[0] = dt.second
        buf[1] = dt.minute
        buf[2] = dt.hour
        buf[3] = dt.day
        buf[4] = dt.month - 1
        buf[5] = dt.year - 1900
       



        pdb.set_trace()
        return buf


from setuptools import setup
import setuptools
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
	name="EasyS7",
	version='1.1.7',
	author='Caner Karagüler',
	author_email='caner.karaguler@gmail.com',
	long_description = long_description,
	long_description_content_type="text/markdown",
	description = 'Easy way to communicate with S7 series PLC.',
	py_modules=["PLC","DataBlock","Utility"],
	packages=setuptools.find_packages(),
	url="https://github.com/canerkaraguler/EasyS7",
	classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],

	)


#-*- coding: utf-8 -*-
import struct
import snap7
from snap7 import util
import urllib
import os
import time
import pdb


def getItemsFromDbLayout(path):
    itemlistt=[]
    with open(path,'r',encoding="utf-8") as fileObject :
        for line in fileObject:
            itemlistt.append(line)

    deliminator='\t'

    items = [
            {
            "name":x.split(deliminator)[0],
            "datatype":x.split(deliminator)[1],
            "bytebit":x.split(deliminator)[2]
            } for x in itemlistt
    ]


    return items



def getDbSize(array,bytekey,datatypekey):
    offsets = { "Bool":1,"Int": 2,"Real":4,"Time":4,"DInt":4,"UDInt":4}#yeni offset degerleri eklenicek
    seq,length = [x[bytekey] for x in array],[x[datatypekey] for x in array]

    maximum=max(seq, key=float)

    idx = seq.index(maximum)
    if length[idx].startswith('String'):

        lastByte = int(maximum.split('.')[0])+(int(length[idx].split('[')[1][:-1]))

    else:

        lastByte = int(maximum.split('.')[0])+(offsets[length[idx]])
    #print(lastByte)
    return lastByte












import struct
import snap7
import random

class DTdint:
    def __init__(self, readBuffer):
        self.readBuffer=readBuffer


    def db_readDint(self,byteArray,index):
        data=byteArray[index:(index+4)]
        value=struct.unpack('>l' ,struct.pack('4B',*data))[0]
        return value

    def db_readUDint(self,byteArray,index):
        data=byteArray[index:(index+4)]
        value=struct.unpack('>L' ,struct.pack('4B',*data))[0]
        return value

    def readValue (self,offset):        
        
        
        result=self.db_readDint(self.readBuffer,offset)
        return result

    def readValueU (self,offset):       
        
        
        result=self.db_readDint(self.readBuffer,offset)
        return result

    def set_dint(self,_bytearray,byte_index,data):
        
        
        dint = int(data)
        _bytes = struct.unpack('4B', struct.pack('>l', dint))
        for i, b in enumerate(_bytes):
            _bytearray[byte_index + i] = b
        return _bytearray


    def set_udint(self,_bytearray,byte_index,data):
        
        
        dint = int(data)
        _bytes = struct.unpack('4B', struct.pack('>L', dint))
        for i, b in enumerate(_bytes):
            _bytearray[byte_index + i] = b
        return _bytearray





import struct
import snap7

class DTstring:
    def __init__(self):
        pass


    def set_string(self,_bytearray, byte_index, value, max_size):
        """
        Set string value
        :params value: string data
        :params max_size: max possible string size
        """
        assert isinstance(value, str)

        size = len(value)
        # FAIL HARD WHEN trying to write too much data into PLC
        if size > max_size:
            raise ValueError(f'size {size} > max_size {max_size} {value}')
        # set len count on first position
        _bytearray[byte_index + 1] = len(value)
        
        i = 0
        # fill array which chr integers
        for i, c in enumerate(value):
            _bytearray[byte_index + 2 + i] = ord(c)

        # fill the rest with empty space
        
        for r in range(len(value), _bytearray[byte_index]-2):
            
            _bytearray[byte_index + 2 + r] = ord(' ')
        return _bytearray